/*
This file is part of the Remote ImageJ.
 
(c) 2011 INSERM
This software is developed at Montpellier RIO Imaging (IFR 122), Montpellier, France (www.mri.cnrs.fr)
Developer: Volker Baecker (volker.baecker@mri.cnrs.fr) 

The Remote ImageJ allows to run ImageJ macros on a distant machine.

This software is governed by the CeCILL-B license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-B
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-B license and that you accept its terms. 
*/
package fr.cnrs.mri.remoteij.client.rmr;

import java.util.Observable;
import java.util.Observer;

import fr.cnrs.mri.macro.io.IOSettings;
import fr.cnrs.mri.sijame.connection.SIJAMEClient;
import fr.cnrs.mri.sijame.connection.SIJAMEServer;
import fr.cnrs.mri.sijame.messages.AnswerMessage;
import fr.cnrs.mri.sijame.messages.MessageQueue;
import fr.cnrs.mri.sijame.messages.RequestMessage;
import fr.cnrs.mri.util.logging.LoggingUtil;

/**
 * The remote macro runner client allows to execute a macro on a remote machine.
 * The result returned by the macro is sent back to the client.   
 * 
 * @author Volker Baecker
 */
public class RemoteMacroRunnerClient implements Observer {
	/**
	 * The dns-name or ip-address of the server.
	 */
	private String server;
	
	/**
	 * The port on which the server is listening.
	 */
	private int serverPort;
	
	/**
	 * The port on which the client is listening for answers from the server 
	 */
	private int port;
	
	/**
	 * The last result that has been sent back from a server.
	 */
	private String lastResult;
	
	/**
	 * The messaging server used for the communication.
	 */
	private SIJAMEServer sijameServer;
	
	/**
	 * The messaging client used for the communication.
	 */
	private SIJAMEClient sijameClient;

	/**
	 * Answer the dns-name or ip-address of the server.
	 * 
	 * @return 	The dns-name or ip-address of the server.
	 */
	public String getServer() {
		return server;
	}
	
	/**
	 * Set the dns-name or ip-address of the server.
	 * 
	 * @param ipAddress The dns-name or ip-address of the server.
	 */
	public void setServer(String ipAddress) {
		this.server = ipAddress;
	}

	/**
	 * Answer the port on which the server is listening.
	 * 
	 * @return		The port on which the server is listening.
	 */
	public int getServerPort() {
		return serverPort;
	}
	
	/**
	 * Set the port on which the server is listening.
	 * 
	 * @param serverPort	The port on which the server is listening.
	 */
	public void setServerPort(int serverPort) {
		this.serverPort = serverPort;
	}

	/**
	 * Answer the port on which the client is listening for answers from the server.
	 *  
	 * @return	The port on which the client is listening for answers from the server. 
	 */
	public int getPort() {
		return port;
	}
	
	/**
	 * Set the port on which the client is listening for answers from the server
	 *  
	 * @param port		The port on which the client is listening for answers from the server. 
	 */
	public void setPort(int port) {
		this.port = port;
	}

	/**
	 * Set the last result back to null.
	 */
	public void resetLastResult() {
		this.lastResult = null;
	}

	/**
	 * Answer the last result that has been sent back from a server.
	 * 
	 * @return	The last result that has been sent back from a server.
	 */
	public String getLastResult() {
		return lastResult;
	}
	
	/**
	 * Run the given macro on the configured server and listen
	 * for the answer.
	 * 
	 * @param macro		The macro that will be executed on the remote machine.
	 */
	public boolean runMacro(String macro) {
		this.resetLastResult();
		if (!this.setupMessaging()) return false;
		if (!this.sendMacroMessage(macro)) {
			this.sijameServer.shutdown();
			while (this.sijameServer.isRunning())
				try {
					Thread.sleep(10);
				} catch (InterruptedException e) {
					LoggingUtil.getLoggerFor(this).warning(LoggingUtil.getMessageAndStackTrace(e));
				}
			return false;
		}
		return true;
	}

	/**
	 * Observe the answer queue for answers from the server.
	 */
	@Override
	public void update(Observable sender, Object aspect) {
		if (sender != this.sijameServer.getAnswerQueue()) return;
		if (aspect.equals(MessageQueue.Aspect.CONTENT)) return;
		AnswerMessage answer = (AnswerMessage) aspect;
		this.handleAnswer(answer);
	}

	/**
	 * Create a sijame-request-message with the macro as data.
	 * 
	 * @param macro		the macro that shall be sent to the server
	 * @return			A sijame-request-message with the macro as data.
	 */
	private boolean sendMacroMessage(String macro) {
		MacroWithSettings macroWithSettings = new MacroWithSettings(macro, IOSettings.getInstance());
		RequestMessage request = new RequestMessage(macroWithSettings);
		request.setReceiver("DistantMacroRunner");
		request.setSenderIPAddress(SIJAMEClient.getIpAddress());
		request.setSenderName("MacroRunnerClient");
		request.setSenderPort(this.getPort());
		request.setTopic("executeMacro");
		boolean result = this.sijameClient.send(request);
		return result;
	}
	
	/**
	 * Start the sijame-server that can receive answer messages from the remote-macro-runner-server.
	 * Configure a sijame-client to send requests to a remote-macro-runner-server.
	 * 
	 * @return	True if the messaging has successfully been setup.
	 */
	private boolean setupMessaging() {
		boolean result = true;
		try {
			sijameServer = new SIJAMEServer(this.getPort());
			sijameServer.start();
			int count = 0;
			while(!sijameServer.isRunning() && count<101) {
				count++;
				Thread.sleep(10);	
			}
			if (!sijameServer.isRunning()) return false;
			sijameClient = new SIJAMEClient(this.getServer(), this.getServerPort());
			sijameServer.getAnswerQueue().addObserver(this);
		} catch (InterruptedException e) {
			result = false;
			LoggingUtil.getLoggerFor(this).severe(LoggingUtil.getMessageAndStackTrace(e));
		}
		return result;
	}
	
	/**
	 * Set the last result to the content of the answer and shutdown the 
	 * sijame-server.
	 * 
	 * @param answer
	 */
	private void handleAnswer(AnswerMessage answer) {
		this.lastResult = (String)answer.getData();
		answer.consume();
		sijameServer.shutdown();
		while (sijameServer.isRunning())
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
				LoggingUtil.getLoggerFor(this).severe(LoggingUtil.getMessageAndStackTrace(e));
			}
	}
}
