/*
This file is part of the Remote ImageJ.
 
(c) 2011 INSERM
This software is developed at Montpellier RIO Imaging (IFR 122), Montpellier, France (www.mri.cnrs.fr)
Developer: Volker Baecker (volker.baecker@mri.cnrs.fr) 

The Remote ImageJ allows to run ImageJ macros on a distant machine.

This software is governed by the CeCILL-B license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-B
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-B license and that you accept its terms. 
*/
package fr.cnrs.mri.remoteij.queueManager;

import java.util.Observable;

import fr.cnrs.mri.server.Server;
import fr.cnrs.mri.sijame.connection.SIJAMEServer;
import fr.cnrs.mri.sijame.messages.MessageQueue;
import fr.cnrs.mri.sijame.messages.RequestMessage;
import fr.cnrs.mri.util.logging.LoggingUtil;

public class QueueManager extends Observable  implements Server {

	private static final String QUEUE_FILENAME = "queue.seq";
	private SIJAMEServer sijameServer;
	private JobDispatcher jobDispatcher;
	private MessageQueue runningQueue;

	public QueueManager(int port) {
		this.sijameServer = SIJAMEServer.currentForPort(port);
		this.sijameServer.addObserver(this);
	}
	
	public static void main(String[] args) {
		if (args.length != 1) {
			System.err.println("Usage: java QueueManager.class port_number]");
			System.exit(1);
		}
		int port = Integer.parseInt(args[0]);
		QueueManager queueManager = new QueueManager(port);
		queueManager.start();
	}
	
	@Override
	public boolean isRunning() {
		return sijameServer.isRunning();
	}

	@Override
	public void start() {
		if (this.isRunning()) return;
		this.sijameServer.start();
		while(!sijameServer.isRunning()) {
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
				LoggingUtil.getLoggerFor(this).warning(LoggingUtil.getMessageAndStackTrace(e));
			}
		}
		this.sijameServer.getRequestQueue().load(this.getQueueFilename());
		this.sijameServer.getRequestQueue().addObserver(this);
		this.sijameServer.getAnswerQueue().addObserver(this);
		this.jobDispatcher = new JobDispatcher(this.sijameServer.getRequestQueue(), this.getRunningQueue());
		(new Thread(this.jobDispatcher)).start();
	}

	private MessageQueue getRunningQueue() {
		if (this.runningQueue == null) this.runningQueue = new MessageQueue();
		return this.runningQueue;
	}

	@Override
	public void stop() {
		if (jobDispatcher != null) jobDispatcher.stop();
		this.sijameServer.shutdown();
		while(sijameServer.isRunning()) {
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
				LoggingUtil.getLoggerFor(this).warning(LoggingUtil.getMessageAndStackTrace(e));
			}
		}
	}

	/**
	 * Look for incoming requests.
	 */
	@Override
	public void update(Observable sender, Object aspect) {
		if (sender==this.sijameServer && aspect.equals(SIJAMEServer.Aspect.IS_RUNNING)) this.handleIsRunningChanged();
		if (sender != sijameServer.getRequestQueue()) return;
		if (aspect.equals(MessageQueue.Aspect.CONTENT)) {this.handleQueueChanged();	return;}
		RequestMessage request = (RequestMessage) aspect;
		if (request.getTopic().equals("executeMacro")) this.handleExecuteMacroRequest(request);
	}

	

	private void handleQueueChanged() {
		this.sijameServer.getRequestQueue().saveAs(this.getQueueFilename());
	}

	public String getQueueFilename() {
		return QUEUE_FILENAME;
	}

	private void handleExecuteMacroRequest(RequestMessage request) {
		// TODO Forward it to a free server using the rmr-client
		
	}

	private void handleIsRunningChanged() {
		this.changed(Server.Aspect.IS_RUNNING);
	}
	
	private void changed(Aspect anAspect) {
		this.setChanged();
		this.notifyObservers(anAspect);
		this.clearChanged();
	}
}
