/*
This file is part of the Remote ImageJ.
 
(c) 2011 INSERM
This software is developed at Montpellier RIO Imaging (IFR 122), Montpellier, France (www.mri.cnrs.fr)
Developer: Volker Baecker (volker.baecker@mri.cnrs.fr) 

The Remote ImageJ allows to run ImageJ macros on a distant machine.

This software is governed by the CeCILL-B license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-B
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-B license and that you accept its terms. 
*/
package fr.cnrs.mri.remoteij.server;

import java.io.IOException;
import java.util.Date;
import java.util.Observable;
import java.util.Observer;
import java.util.logging.FileHandler;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;

import fr.cnrs.mri.server.Server;

public abstract class ServerConsole extends Observable implements Observer {
	public enum Status {DOWN, RUNNING};
	public enum Aspect {STATUS, START_TIME, STOP_TIME};
	
	protected Server server;
	protected Status status = Status.DOWN;
	protected Date started;
	protected Date stopped;
	
	public ServerConsole() {
		try {
			FileHandler handler = new FileHandler(this.name()+ "_%g.log", this.logFileSizeLimit(), this.numberOfLogFiles(), true);
			handler.setFormatter(new SimpleFormatter());
			Logger logger = Logger.getLogger(this.subsystemToLog());
			logger.addHandler(handler);
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	abstract public String name();
	abstract public int logFileSizeLimit();
	abstract public int numberOfLogFiles();
	abstract public String subsystemToLog();
	abstract protected ServerView getView();
	abstract protected Server getNewServer();
	abstract protected ServerConfig getConfig();
	
	public void show() {
		this.getView().setVisible(true);
	}

	public int getPort() {
		return this.getConfig().getPort();
	}
	
	public void setPort(int port) {
		this.getConfig().setPort(port);
	}

	public Status getStatus() {
		return status;
	}
	
	public void setStatus(Status status) {
		this.status = status;
		if (status.equals(Status.DOWN)) this.setStopped(new Date());
		if (status.equals(Status.RUNNING)) {
			this.setStarted(new Date());
			this.setStopped(null);
		}
		this.changed(Aspect.STATUS);
	}

	protected void changed(Aspect anAspect) {
		this.setChanged();
		this.notifyObservers(anAspect);
		this.clearChanged();
	}

	@Override
	public void update(Observable sender, Object aspect) {
		if (aspect.equals(Server.Aspect.IS_RUNNING)) this.handleIsRunningChanged();
	}

	protected void handleIsRunningChanged() {
		if (server==null || !server.isRunning()) this.setStatus(Status.DOWN);
		else this.setStatus(Status.RUNNING);
	}

	public Date getStarted() {
		return started;
	}

	public void setStarted(Date date) {
		this.started = date;
		this.changed(Aspect.START_TIME);
	}

	public Date getStopped() {
		return stopped;
	}
	
	public void setStopped(Date date) {
		this.stopped = date;
		this.changed(Aspect.STOP_TIME);
	}

	public void startOrStop() {
		if (server==null || !server.isRunning()) {
			if (server!=null) server.deleteObserver(this);
			server = this.getNewServer();
			server.addObserver(this);
			this.setStatus(Status.RUNNING);
			server.start();
		}
		else {
			server.stop();
			this.setStatus(Status.DOWN);
		}
	}
}
