/*
This file is part of the Remote ImageJ.
 
(c) 2011 INSERM
This software is developed at Montpellier RIO Imaging (IFR 122), Montpellier, France (www.mri.cnrs.fr)
Developer: Volker Baecker (volker.baecker@mri.cnrs.fr) 

The Remote ImageJ allows to run ImageJ macros on a distant machine.

This software is governed by the CeCILL-B license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-B
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-B license and that you accept its terms. 
*/
package fr.cnrs.mri.remoteij.server.rfs;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Observable;
import javax.swing.filechooser.FileSystemView;

import fr.cnrs.mri.files.RemoteFile;
import fr.cnrs.mri.server.Server;
import fr.cnrs.mri.sijame.connection.MessageHandler;
import fr.cnrs.mri.sijame.connection.MessageSource;
import fr.cnrs.mri.sijame.connection.SIJAMEServer;
import fr.cnrs.mri.sijame.messages.Message;
import fr.cnrs.mri.util.logging.LoggingUtil;

public class RemoteFilesystemServer extends Observable implements MessageHandler, Server {
	private MessageSource messageSource;
	private SIJAMEServer sijameServer;

	public RemoteFilesystemServer(int port) {
		this.sijameServer = SIJAMEServer.currentForPort(port);
		this.sijameServer.addObserver(this);
		this.sijameServer.addMessageHandler(this);
	}
	
	public static void main(String[] args) {
		if (args.length != 1) {
			System.err.println("Usage: java RemoteFilesystemServer port_number]");
			System.exit(1);
		}
		int port = Integer.parseInt(args[0]);
		RemoteFilesystemServer server = new RemoteFilesystemServer(port);
		server.start();
	}
	
	/**
	 * Start the server and start observing the request-queue.
	 */
	@Override
	public void start() {
		if (this.isRunning()) return;
		this.sijameServer.start();
		while(!sijameServer.isRunning()) {
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
				LoggingUtil.getLoggerFor(this).warning(LoggingUtil.getMessageAndStackTrace(e));
			}
		}
	}
	
	/**
	 * Stop the server.
	 */
	@Override
	public void stop() {
		this.sijameServer.shutdown();
		while(sijameServer.isRunning()) {
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
				LoggingUtil.getLoggerFor(this).warning(LoggingUtil.getMessageAndStackTrace(e));
			}
		}
	}
	
	@Override
	public boolean isRunning() {
		return sijameServer.isRunning();
	}
	
	@Override
	public Object handleMessage(Message message) {
		Object[] parts = (Object[]) message.getData();
		String methodName = (String) parts[0];  
		Object result = SIJAMEServer.getUnhandled();
		FileSystemView filesystemView = FileSystemView.getFileSystemView();
		if (methodName.equals("ping")) result = "pong";
		if (methodName.equals("createFileObject") && parts.length==3) result = RemoteFile.from(filesystemView.createFileObject((File)parts[1], (String)parts[2]));
		if (methodName.equals("createFileObject") && parts.length==2) result = RemoteFile.from(filesystemView.createFileObject((String)parts[1]));
		if (methodName.equals("getChild")) result = RemoteFile.from(filesystemView.getChild((File)parts[1], (String)parts[2]));
		if (methodName.equals("getDefaultDirectory")) result = RemoteFile.from(this.getDefaultDirectoryFor(message.getSenderName()));
		if (methodName.equals("getFiles")) result = this.getFiles((File)parts[1], (Boolean)parts[2], filesystemView);
		if (methodName.equals("getHomeDirectory")) result = RemoteFile.from(this.getDefaultDirectoryFor(message.getSenderName()));
		if (methodName.equals("getParentDirectory")) result = RemoteFile.from(filesystemView.getParentDirectory((File)parts[1]));
		if (methodName.equals("getRoots")) result = this.getRoots(message.getSenderName());
		if (methodName.equals("getSystemDisplayName")) result = filesystemView.getSystemDisplayName((File)parts[1]);
		if (methodName.equals("getSystemTypeDescription")) result = filesystemView.getSystemTypeDescription((File)parts[1]);
		if (methodName.equals("isFileSystem")) result = filesystemView.isFileSystem((File)parts[1]);
		if (methodName.equals("isFileSystemRoot")) result = this.isFileSystemRoot((File)parts[1], message.getSenderName());
		if (methodName.equals("isHiddenFile")) result = filesystemView.isHiddenFile((File)parts[1]);
		if (methodName.equals("isParent")) result = filesystemView.isParent((File)parts[1], (File)parts[2]);
		if (methodName.equals("isRoot")) result = this.isFileSystemRoot((File)parts[1], message.getSenderName());
		if (methodName.equals("isTraversable")) result = filesystemView.isTraversable((File)parts[1]);
		if (methodName.equals("createNewFolder"))
			try {
				result = RemoteFile.from(filesystemView.createNewFolder((File)parts[1]));
			} catch (IOException e) {
				LoggingUtil.getLoggerFor(this).warning(LoggingUtil.getMessageAndStackTrace(e));
			}
		return result;
	}

	private File[] getFiles(File file, Boolean hidden, FileSystemView filesystemView) {
		File[] files = filesystemView.getFiles(file, hidden);
		RemoteFile[] remoteFiles = new RemoteFile[files.length];
		int index = 0;
		for (File aFile : files) {
			remoteFiles[index] = RemoteFile.from(aFile);
			index++;
		}
		return remoteFiles;
	}

	private boolean isFileSystemRoot(File file, String username) {
		boolean result = false;
		result = Arrays.asList(getRoots(username)).contains(file);
		return result;
	}
	
	private File[] getRoots(String senderName) {
		RFSServerConfig config = RFSServerConfig.getCurrent();
		File[] roots = null;
		if (RFSServerConfig.getCurrent().getUseUserFolders()) {
			roots = new File[2];
			roots[0] = RemoteFile.from(this.getDefaultDirectoryFor(senderName));
			roots[1] = RemoteFile.from(this.getDefaultDirectoryFor("common"));
		} else {
			roots = new File[1];
			roots[0] = RemoteFile.from(new File(config.getFileserverRoot()));
		}
		return roots;
	}

	private File getDefaultDirectoryFor(String senderName) {
		RFSServerConfig config = RFSServerConfig.getCurrent();
		String fileserverRoot = config.getFileserverRoot();
		boolean useUserFolders = config.getUseUserFolders();
		if (!useUserFolders) return new File(fileserverRoot);
		File dir = new File(fileserverRoot);
		File userDir = FileSystemView.getFileSystemView().createFileObject(dir, senderName);
		if (!userDir.exists()) userDir.mkdir();
		return userDir;
	}

	@Override
	public void setMessageSource(MessageSource messageSource) {
		this.messageSource = messageSource;
	}

	@Override
	public boolean wantsToHandle(Message message) {
		return message.getReceiver().equals("remote-filesystem-server");
	}
	
	public MessageSource getMessageSource() {
		return messageSource;
	}

	@Override
	public void update(Observable sender, Object aspect) {
		if (sender==this.sijameServer && aspect.equals(SIJAMEServer.Aspect.IS_RUNNING)) this.handleIsRunningChanged();
	}
	
	private void handleIsRunningChanged() {
		this.changed(Aspect.IS_RUNNING);
	}
	
	private void changed(Aspect anAspect) {
		this.setChanged();
		this.notifyObservers(anAspect);
		this.clearChanged();
	}
}
