/*
 This file is part of the Simple Java Message Exchange (SIJAME) software.

(c) 2011 INSERM
SIJAME is developed at Montpellier RIO Imaging (IFR 122), Montpellier, France (www.mri.cnrs.fr)
Developer: Volker Bäcker (volker.baecker@mri.cnrs.fr) 

This software is a very simple message oriented middleware (mom). It allows 
a communication between programms via sockets. A software (client) can send a 
message to another software on another machine (server). The message is 
put into a message queue. Whenever the currently first message in the queue changes
a notification is sent to observers of the queue.  

This software is governed by the CeCILL-B license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-B
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-B license and that you accept its terms. 
 */
package fr.cnrs.mri.sijame.connection;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.util.logging.Logger;
import fr.cnrs.mri.sijame.messages.Message;
import fr.cnrs.mri.util.logging.LoggingUtil;

/**
 * The communication processor reads a message from the client and 
 * pushes it into a message queue.
 * 
 * @author baecker
 *
 */
public class CommunicationProcessor implements Runnable {

	/**
	 * The communicationServer we are running inside
	 */
	private SIJAMEServer server;

	/**
	 * The socket connected to the client
	 */
	private Socket socket;

	/**
	 * the server's input stream
	 */
	private ObjectInputStream objectIn;

	/**
	 * the server's output stream
	 */
	private ObjectOutputStream objectOut;

	/**
	 * The logger used by the communication processor.
	 */
	private Logger logger;

	/**
	 * Create a new CommunicationProcessor
	 */
	public CommunicationProcessor(SIJAMEServer server, Socket socket ) {
		this.server = server;
		this.socket = socket;
		logger = LoggingUtil.getLoggerFor(this);
		new Thread( this ).start();
	}

	/**
	 * Read one object from the incoming stream and send a receipt text to
	 * the client. The object is supposed to be a message. It will be pushed
	 * into the request or answer queue. 
	 */
	public void run() {
		this.setupSockets();
		try {
			logger.info("Got connection from socket "+socket );
			while(true) {
				Message message = (Message) objectIn.readObject();
				if (message.isCloseMessage()) break;
				Object answer = server.notifyMessageHandlers(message);
				boolean hasBeenHandled = answer==null || !answer.equals(SIJAMEServer.getUnhandled());
				if (!hasBeenHandled) answer = Message.receiptText();
				if (answer==null) answer = "null";
				objectOut.writeObject(answer);
				objectOut.flush();
				if (!hasBeenHandled) this.processMessage( message );
				if (!message.isKeepTheLine()) break;
			}
		} catch (IOException e) {
			logger.warning(LoggingUtil.getMessageAndStackTrace(e));
		} catch (ClassNotFoundException e) {
			logger.warning(LoggingUtil.getMessageAndStackTrace(e));
		} finally {
			this.closeStreams();
			server.removeConnection( this );
			logger.info("Closed connection from socket "+socket );
		}
	}

	/**
	 * Process a received message. If the message is a request, it is pushed into the request queue.
	 * If the message is an answer it is pushed into the answer queue.
	 * 
	 * @param message	the received message
	 */
	private void processMessage( Message message ) {
		if (message.isRequest()) server.getRequestQueue().push(message);
		else server.getAnswerQueue().push(message);
	}


	private void setupSockets() {
		OutputStream out;
		try {
			out = socket.getOutputStream();
			InputStream in = socket.getInputStream();
			objectOut = new ObjectOutputStream( out );
			objectIn = new ObjectInputStream( in );
		} catch (IOException e) {
			logger.warning(LoggingUtil.getMessageAndStackTrace(e));
			this.closeStreams();
		}
	}

	/**
	 * Close the streams and the socket that have been used in this communication.
	 */ 
	protected void closeStreams() {
		try {
			if (!socket.isOutputShutdown()) socket.shutdownOutput();
		} catch (IOException e) {
			logger.info( "Error closing socket "+socket );
			logger.warning(LoggingUtil.getMessageAndStackTrace(e));
		}
	}

}
