/*
 This file is part of the Simple Java Message Exchange (SIJAME) software.

(c) 2011 INSERM
SIJAME is developed at Montpellier RIO Imaging (IFR 122), Montpellier, France (www.mri.cnrs.fr)
Developer: Volker Baecker (volker.baecker@mri.cnrs.fr) 

This software is a very simple message oriented middleware (mom). It allows 
a communication between programms via sockets. A software (client) can send a 
message to another software on another machine (server). The message is 
put into a message queue. Whenever the currently first message in the queue changes
a notification is sent to observers of the queue.  

This software is governed by the CeCILL-B license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-B
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-B license and that you accept its terms. 
 */

package fr.cnrs.mri.sijame.connection;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.Socket;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import fr.cnrs.mri.sijame.messages.CloseMessage;
import fr.cnrs.mri.sijame.messages.Message;
import fr.cnrs.mri.util.logging.LoggingUtil;


/**
 * The communication client can be used to send a message to a server.
 * It is creates for a specific host and port. To send a message it creates
 * a socket that is closed immediately after a receipt for the message has 
 * been received. 
 * 
 * @author Volker Baecker
 */
public class SIJAMEClient {
	/**
	 * A default instance of the client. If only one instance is needed the software using the client
	 * doesn't have to keep its own reference to the client.   
	 */
	private static SIJAMEClient instance;
	
	private static Map<String, SIJAMEClient> clients;
	/**
	 * The ip-address or hostname of the server to which the client shall send a message.
	 */
	private String host;
	/**
	 * The port on which the server is listening.
	 */
	private int port;
	
	private boolean keepTheLine = false;

	private Socket socket;

	private ObjectOutputStream objectOut;

	private ObjectInputStream objectIn;

	/**
	 * Create a client for a communication with the given host using the given port.
	 * 
	 * @param host		The host to which messages shall be send.
	 * @param port		The port on which the host is listening.
	 */
	public SIJAMEClient(String host, int port) {
		this.host = host;
		this.port = port;
		getClients().put(host+":" + port, this);
	}

	/**
	 * Create a new default instance, that can be accessed using getInstance.
	 * 
	 * @param host	The host to which messages shall be send.
	 * @param port  The port on which the host is listening.
	 * @return	A communication client that is setup for a communication with the given server on the given port.
	 */
	public static SIJAMEClient currentForHostAndPort(String host, int port) {
		String key = host+ ":" + port;
		if (!getClients().containsKey(key)) new SIJAMEClient(host, port);
		SIJAMEClient result = getClients().get(key);
		instance = result;
		return result;
	}
	
	/**
	 * Answser the default instance of the communication client. The default instance is null if it hasn't been
	 * created with currentForHostAndPort before.
	 * 
	 * @return	The default instance of the communication client
	 */
	public static SIJAMEClient getCurrent() {
		return instance; 
	}
	
	/**
	 * Answer the hostname or ip-address of the server
	 * 
	 * @return  The hostname or ip-address of the server
	 */
	public String getHost() {
		return host;
	}

	/**
	 * The port on which the client will contact the server.
	 * 
	 * @return	The port on which the client will contact the server.
	 */
	public int getPort() {
		return port;
	}

	/**
	 * Send the message to the server. Answer true if the server sends a receipt for the message
	 * and false otherwise.
	 *  
	 * @param aMessage		The message that is send to the server
	 * @return				true if the server answers a receipt and false 
	 * 						if the server answers something else than the predefined receipt or 
	 * 						an unknown host or io exception occured.
	 */
	public boolean send(Message aMessage) {
		boolean result = false;
		String answer = (String) sendAndGetAnswer(aMessage);
		if (answer==null) return result;
		if (answer.equals(Message.receiptText())) result = true;
		return result;
	}
	
	synchronized public Object sendAndGetAnswer(Message aMessage) {
		 Object result = null;
			try {
				if (!this.setupSocketAndSteams()) return false;
				if (isKeepTheLine()) aMessage.setKeepTheLine(true);
				objectOut.writeObject(aMessage);
				objectOut.flush();
			  	result = objectIn.readObject();
			  	if (result.equals("null")) result = null;
			  	if (!this.keepTheLine) this.hangUp();
			} catch (UnknownHostException e) {
				LoggingUtil.getLoggerFor(this).warning(LoggingUtil.getMessageAndStackTrace(e));
			} catch (IOException e) {
				LoggingUtil.getLoggerFor(this).warning(LoggingUtil.getMessageAndStackTrace(e));
			} catch (ClassNotFoundException e) {
				LoggingUtil.getLoggerFor(this).warning(LoggingUtil.getMessageAndStackTrace(e));
			}
		return result;
	}
	
	private boolean setupSocketAndSteams() throws UnknownHostException, IOException {
		if (isKeepTheLine() && socket!=null && !socket.isClosed() && socket.isConnected()) return true;
		socket = new Socket(host, port);
		if (socket.isInputShutdown()) return false;
		if (socket.isOutputShutdown()) return false;
		OutputStream out = socket.getOutputStream();
		InputStream in = socket.getInputStream();
		objectOut = new ObjectOutputStream( out );
		objectIn = new ObjectInputStream( in );
		return true;
	}
	
	public static String getIpAddress() {
		String ipAddress = null;
		try {
			NetworkInterface.getNetworkInterfaces();
			Enumeration<NetworkInterface> e = NetworkInterface.getNetworkInterfaces();
			while(e.hasMoreElements()) {
				NetworkInterface ni = (NetworkInterface) e.nextElement();
				Enumeration<InetAddress> e2 = ni.getInetAddresses();
				while (e2.hasMoreElements()){
					InetAddress ip = (InetAddress) e2.nextElement();
					if (ip.isLoopbackAddress()) continue;
					if (ip.toString().split("\\.").length!=4) continue;
					ipAddress = ip.getHostAddress();
					break;
				}
			}
		} catch (SocketException e) {
			LoggingUtil.getLoggerFor(SIJAMEClient.class).warning(LoggingUtil.getMessageAndStackTrace(e));
		}
		return ipAddress;
	}
	
	static private Map<String, SIJAMEClient> getClients() {
		if (clients==null) clients = new HashMap<String, SIJAMEClient>();
		return clients;
	}

	public void setKeepTheLine(boolean keepTheLine) {
		this.keepTheLine = keepTheLine;
	}

	public boolean isKeepTheLine() {
		return keepTheLine;
	}
	
	public void hangUp() {
		try {
			if (this.isKeepTheLine()) {
				Message message = new CloseMessage("close");
				objectOut.writeObject(message);
				objectOut.flush();
			}
			objectIn.close();
			objectOut.close();
		  	socket.close();
		} catch (IOException e) {
			LoggingUtil.getLoggerFor(this).warning(LoggingUtil.getMessageAndStackTrace(e));
		}
	}
}
