package tests.fr.cnrs.mri.remoteij.client.rfs;

import static org.junit.Assert.*;

import java.io.File;
import java.io.IOException;

import javax.swing.Icon;
import javax.swing.filechooser.FileSystemView;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import fr.cnrs.mri.remoteij.client.rfs.RemoteFilesystemView;
import fr.cnrs.mri.remoteij.server.rfs.RemoteFilesystemServer;
import fr.cnrs.mri.sijame.connection.SIJAMEClient;
import fr.cnrs.mri.testData.TestConfig;

public class RemoteFilesystemViewTest {

	private static RemoteFilesystemServer server;
	private static RemoteFilesystemView filesystemView;

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		server = new RemoteFilesystemServer(4000);
		server.start();
		filesystemView = new RemoteFilesystemView(SIJAMEClient.getIpAddress(), "4000");
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		filesystemView.close();
		server.stop();
	}

	@Test
	public void testCreateFileObjectFileString() {
		File dir = new File(TestConfig.testFolder);
		File file = new File(TestConfig.testFolder + "test.txt");
		File aFile = filesystemView.createFileObject(dir, "test.txt");
		assertTrue(aFile.equals(file));
	}

	@Test
	public void testCreateFileObjectString() {
		File file = new File(TestConfig.testFolder + "test.txt");
		File aFile = filesystemView.createFileObject(TestConfig.testFolder + "test.txt");
		assertTrue(aFile.equals(file));
	}

	@Test
	public void testGetChildFileString() {
		File file = new File(TestConfig.testFolder);
		String filename = "test.txt";
		File expected = new File(TestConfig.testFolder + "test.txt");
		File aFile = filesystemView.getChild(file, filename);
		assertEquals(expected, aFile);
	}

	@Test
	public void testGetDefaultDirectory() {
		File dir = filesystemView.getDefaultDirectory();
		assertTrue(dir.exists());
	}

	@Test
	public void testGetFilesFileBoolean() throws IOException {
		File dir = filesystemView.getDefaultDirectory();
		File[] files = filesystemView.getFiles(dir, true);
		assertEquals(0, files.length);
		File a = FileSystemView.getFileSystemView().createFileObject(dir, "a.txt");
		File b = FileSystemView.getFileSystemView().createFileObject(dir, "b.txt");
		File c = FileSystemView.getFileSystemView().createFileObject(dir, "c.txt");
		a.createNewFile();
		b.createNewFile();
		c.createNewFile();
		files = filesystemView.getFiles(dir, true);
		assertEquals(3, files.length);
		assertTrue(files[0].getAbsolutePath().equals(c.getAbsolutePath()));
		assertTrue(files[1].getAbsolutePath().equals(b.getAbsolutePath()));
		assertTrue(files[2].getAbsolutePath().equals(a.getAbsolutePath()));
		a.delete();
		b.delete();
		c.delete();
		
	}

	@Test
	public void testGetHomeDirectory() {
		File dir = filesystemView.getHomeDirectory();
		assertTrue(dir.exists());
	}

	@Test
	public void testGetParentDirectoryFile() {
		File home = filesystemView.getDefaultDirectory();
		File a = FileSystemView.getFileSystemView().createFileObject(home, "/a");
		a.mkdir();
		File result = filesystemView.getParentDirectory(a);
		assertEquals(home, result);
		a.delete();
	}

	@Test
	public void testGetRoots() {
		File[] roots = filesystemView.getRoots();
		assertTrue(roots.length>0);
		for (File file : roots) {
			assertTrue(file.exists());
		}
	}

	@Test
	public void testGetSystemDisplayNameFile() {
		File dir = filesystemView.getDefaultDirectory();
		String displayName = filesystemView.getSystemDisplayName(dir);
		assertEquals(dir.getName(), displayName);
	}

	@Test
	public void testGetSystemIconFile() {
		File dir = filesystemView.getDefaultDirectory();
		Icon icon = filesystemView.getSystemIcon(dir);
		assertEquals(FileSystemView.getFileSystemView().getSystemIcon(dir), icon);
	}

	@Test
	public void testGetSystemTypeDescriptionFile() {
		File dir = filesystemView.getDefaultDirectory();
		String description = filesystemView.getSystemTypeDescription(dir);
		assertNull(description);
	}

	@Test
	public void testIsComputerNodeFile() {
		File dir = filesystemView.getDefaultDirectory();
		assertFalse(filesystemView.isComputerNode(dir));
	}

	@Test
	public void testIsDriveFile() {
		File dir = filesystemView.getDefaultDirectory();
		assertFalse(filesystemView.isDrive(dir));
	}

	@Test
	public void testIsFileSystemFile() {
		File dir = filesystemView.getDefaultDirectory();
		assertTrue(filesystemView.isFileSystem(dir));
	}

	@Test
	public void testIsFileSystemRootFile() {
		File dir = filesystemView.getDefaultDirectory();
		File a = FileSystemView.getFileSystemView().createFileObject(dir, "a");
		a.mkdir();
		assertTrue(filesystemView.isFileSystemRoot(dir));
		assertFalse(filesystemView.isFileSystemRoot(a));
		a.delete();
	}

	@Test
	public void testIsFloppyDriveFile() {
		File dir = filesystemView.getDefaultDirectory();
		assertFalse(filesystemView.isFloppyDrive(dir));
	}

	@Test
	public void testIsHiddenFileFile() {
		File dir = filesystemView.getDefaultDirectory();
		File hidden = FileSystemView.getFileSystemView().createFileObject(dir, ".hidden");
		hidden.mkdir();
		assertFalse(filesystemView.isHiddenFile(dir));
		assertTrue(filesystemView.isHiddenFile(hidden));
		hidden.delete();
	}

	@Test
	public void testIsParentFileFile() throws IOException {
		File dir = filesystemView.getDefaultDirectory();
		assertFalse(filesystemView.isParent(dir, dir));
		File a = FileSystemView.getFileSystemView().createFileObject(dir, "a.txt");
		a.createNewFile();
		assertTrue(filesystemView.isParent(dir, a));
		a.delete();
	}

	@Test
	public void testIsRootFile() {
		File dir = filesystemView.getDefaultDirectory();
		File a = FileSystemView.getFileSystemView().createFileObject(dir, "a");
		a.mkdir();
		assertTrue(filesystemView.isRoot(dir));
		assertFalse(filesystemView.isRoot(a));
		a.delete();
	}

	@Test
	public void testIsTraversableFile() {
		File dir = filesystemView.getDefaultDirectory();
		assertTrue(filesystemView.isTraversable(dir));
	}

	@Test
	public void testCreateNewFolderFile() throws IOException {
		File dir = filesystemView.getDefaultDirectory();
		File newFolder = filesystemView.createNewFolder(dir);
		assertTrue(newFolder.exists());
		newFolder.delete();
	}

}
